/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: CoComposePasteAction.java,v 1.2 2004/04/28 12:24:36 dwagelaa Exp $
 * $Log: CoComposePasteAction.java,v $
 * Revision 1.2  2004/04/28 12:24:36  dwagelaa
 * worked on drag-n-drop
 * updated copy/paste
 * updated deep copy operation
 *
 * Revision 1.1  2004/04/27 16:43:58  dwagelaa
 * + added drag-n-drop
 * + added copy/paste
 * + added general copying operation
 * - removed examples (in separate module now)
 *
 */

package be.ac.vub.cocompose.eclipse.actions;

import java.util.List;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.gef.internal.GEFMessages;
import org.eclipse.gef.requests.CreateRequest;
import org.eclipse.gef.requests.CreationFactory;
import org.eclipse.gef.ui.actions.Clipboard;
import org.eclipse.gef.ui.actions.GEFActionConstants;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.ui.IWorkbenchPart;

import be.ac.vub.cocompose.CoCompose;
import be.ac.vub.cocompose.eclipse.CoComposeElementFactory;
import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.VisitedElement;
import be.ac.vub.cocompose.lang.core.RelationElement;
import be.ac.vub.cocompose.operations.CopyElement;

/**
 * Pastes CoCompose model elements.
 * @author Dennis Wagelaar
 * @version 0.2.9, 28/04/2004
 * @since 0.2.9
 * @see be.ac.vub.cocompose.eclipse.actions.CoComposeCopyAction
 */
public class CoComposePasteAction extends SelectionAction
{
  private CoCompose settings = CoCompose.getDefault();

  /**
   * Creates a CoComposePasteAction.
   * @param part the CoCompose editor.
   */
  public CoComposePasteAction(IWorkbenchPart part)
  {
    super(part);
  }

  /**
   * Executes the command returned by {@link #createPasteCommand()}.
   */
  public void run() 
  {
    execute(createPasteCommand());
  }

  /**
   * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
   */
  protected boolean calculateEnabled()
  {
    //Command command = createPasteCommand();
    //return (getClipboardContents() != null && 
    //        command != null && command.canExecute());
    return ((getClipboardContents() != null) &&
            (getSelectedObjects() != null) &&
            (getSelectedObjects().size() == 1) &&
            (getSelectedObjects().get(0) instanceof EditPart));
  }

  /**
   * Creates and returns a command (which may be <code>null</code>)
   * to create a new EditPart based on the template on the clipboard.
   * @return the paste command
   */
  protected Command createPasteCommand() 
  {
    if (!calculateEnabled())
      return null;
    EditPart target = (EditPart) getSelectedObjects().get(0);
    CompoundCommand command = new CompoundCommand();
    List clp = getClipboardContents();
    try
    {
      CopyElement copy = new CopyElement(settings.getFactory());
      List copies = copy.copy(clp);
      for (int i = 0; i < copies.size(); i++)
      {
        VisitedElement element = (VisitedElement) copies.get(i);
        CreateRequest request = new CreateRequest();
        request.setFactory(getFactory(element));
        request.setLocation(getPasteLocation(element));
        command.add(target.getCommand(request));
      }
      return command;
    }
    catch (ModelElementException e)
    {
      settings.getLog().report(e);
    }
    return null;
  }

  /**
   * Returns the contents of the default GEF {@link Clipboard} if it is
   * a {@link java.util.List}.
   * @return the clipboard's contents
   */
  protected List getClipboardContents() 
  {
    Object clp = Clipboard.getDefault().getContents();
    if (clp instanceof List)
      return (List) clp;
    else
      return null;
  }

  /**
   * Returns the appropriate Factory object to be used for the specified 
   * template. This Factory is used on the CreateRequest that is sent to the 
   * target EditPart.
   * @param template the template Object
   * @return a Factory
   */
  protected CreationFactory getFactory(Object template) 
  {
    return new CoComposeElementFactory(
      settings.getFactory(), 
      template);
  }

  /**
   * Get the location where to paste.
   * @param element the element for which to determine the location.
   * @return the location where to paste.
   */
  protected Point getPasteLocation(VisitedElement element)
  {
    Point loc = new Point(10, 10);
    if (element instanceof RelationElement)
    {
      java.awt.Point pos = ((RelationElement) element).getPosition();
      loc.translate(pos.x, pos.y);
    }
    return loc;
  }

  /**
   * @see org.eclipse.gef.ui.actions.EditorPartAction#init()
   */
  protected void init() 
  {
    setId(GEFActionConstants.PASTE);
    setText(GEFMessages.PasteAction_Label);
    setToolTipText(GEFMessages.PasteAction_Tooltip);
  }
}
