/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: EPSGraphicsDebug.java,v 1.3 2004/04/28 16:30:08 dwagelaa Exp $
 * $Log: EPSGraphicsDebug.java,v $
 * Revision 1.3  2004/04/28 16:30:08  dwagelaa
 * rearranged EPS export under ExportAction
 *
 * Revision 1.2  2004/04/26 16:32:38  dwagelaa
 * Added CVS tags
 *
 */

package be.ac.vub.cocompose.eclipse.draw2d.debug;

import java.awt.image.BufferedImage;

import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;

import be.ac.vub.cocompose.CoCompose;
import be.ac.vub.cocompose.eclipse.draw2d.EPSGraphics;
import be.ac.vub.cocompose.log.Log;

/**
 * Debugging code for EPSGraphics
 * @author Dennis Wagelaar
 * @version 0.2.9, 24/04/2004
 * @since 0.2.9
 * @see be.ac.vub.cocompose.eclipse.draw2d.EPSGraphics
 */
public aspect EPSGraphicsDebug
{
  private static Log log = CoCompose.getDefault().getLog();
  
  pointcut toAWTImage(EPSGraphics g, Image srcImage):
    target(g) &&
    args(srcImage) &&
    execution(BufferedImage EPSGraphics.toAWTImage(Image));
  
  pointcut create(EPSGraphics g):
    target(g) &&
    initialization(EPSGraphics.new(..));

  pointcut dispose(EPSGraphics g):
    target(g) &&
    execution(void EPSGraphics.dispose());
    
  pointcut eachmethod(): execution(* EPSGraphics.*(..));
  
  before(EPSGraphics g): create(g)
  {
    beforeCreation(g);
  }
  
  after(EPSGraphics g): dispose(g)
  {
    afterDispose();
  }
  
  BufferedImage around(EPSGraphics g, Image srcImage): toAWTImage(g, srcImage)
  {
    BufferedImage img;
    try
    {
      beforeToAWTImage(g, srcImage);
      img = proceed(g, srcImage);
    }
    catch (Exception e)
    {
      log.log(e.getMessage(), Log.ERROR, e);
      log.endLog(Log.INFO);
      ImageData src = srcImage.getImageData();
      img = new BufferedImage(src.width, src.height, 
        BufferedImage.TYPE_3BYTE_BGR);
    }
    finally
    {
      afterToAWTImage(g, srcImage);
    }
    return img;
  }
  
  before(EPSGraphics g, Color c):
    target(g) &&
    args(c) &&
    execution(* EPSGraphics.*(Color))
  {
    beforeSetColor(g, c);
  }
  
  before(): eachmethod()
  {
    beforeTrace(thisJoinPointStaticPart.getSignature().toString());
  }
  
  after(): eachmethod()
  {
    afterTrace();
  }
  
  /**
   * Performs logging at creation of g.
   * @param g the graphics context.
   */
  public static void beforeCreation(EPSGraphics g)
  {
    log.log("Writing to EPS", Log.INFO, null);
  }

  /**
   * Closes off the logging.
   */  
  public static void afterDispose()
  {
    log.endLog(Log.INFO);
  }

  /**
   * Does logging/introspection before g.toAWTImage().
   * @param g the graphics context.
   * @param srcImage the image to be converted.
   */
  public static void beforeToAWTImage(EPSGraphics g, Image srcImage)
  {
    log.log("Converting SWT Image to AWT Image", Log.INFO, null);
    log.log(srcImage.toString(), Log.INFO, null);
    log.endLog(Log.INFO);
  }

  /**
   * Logs color code.
   * @param g the graphics context.
   * @param c the color to be set.
   */
  public static void beforeSetColor(EPSGraphics g, Color c)
  {
    java.awt.Color awtc =
      new java.awt.Color(c.getRed(), c.getGreen(), c.getBlue());
    log.log("Color set to " + awtc.toString(), Log.INFO, null);
    log.endLog(Log.INFO);
  }

  /**
   * Logs each method call.
   * @param sign the method signature
   */
  public static void beforeTrace(String sign)
  {
    log.log(sign + " invoked", Log.INFO, null);
  }

  /**
   * Ends log of each method call.
   */
  public static void afterTrace()
  {
    log.endLog(Log.INFO);
  }

  /**
   * after g.toAWTImage().
   * @param g the graphics context.
   * @param srcImage the image that is converted.
   */
  public static void afterToAWTImage(EPSGraphics g, Image srcImage)
  {
    log.endLog(Log.INFO);
  }
}
