/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose.eclipse.figures;

import org.eclipse.draw2d.AbstractLabeledBorder;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.graphics.Image;

/**
 * A labeled border for CoCompose concepts.
 * @author Dennis Wagelaar
 * @version 0.2.4, 22/01/2004
 * @since 0.2.0
 */
public class ConceptBorder extends AbstractLabeledBorder
{
  private static final int STD_MARGIN = 2;
  private static final int UL_MARGIN = 6;
  
  private boolean underline = true;
  private int linestyle = Graphics.LINE_SOLID;
  private boolean bold = false;
  private Image icon = null;

  /**
   * @see org.eclipse.draw2d.AbstractLabeledBorder#calculateInsets(org.eclipse.draw2d.IFigure)
   */
  protected Insets calculateInsets(IFigure figure)
  {
    int height = getTextExtents(figure).height;
    if (getUnderline())
      return new Insets(height + UL_MARGIN + STD_MARGIN * 2, UL_MARGIN, UL_MARGIN, UL_MARGIN);
    else
      return new Insets(height, STD_MARGIN, STD_MARGIN * 2, STD_MARGIN);
  }

  /**
   * @see org.eclipse.draw2d.AbstractLabeledBorder#invalidate()
   */
  public void invalidate()
  {
    super.invalidate();
  }

  /**
   * @see org.eclipse.draw2d.Border#paint(org.eclipse.draw2d.IFigure, org.eclipse.draw2d.Graphics, org.eclipse.draw2d.geometry.Insets)
   */
  public void paint(IFigure figure, Graphics g, Insets insets)
  {
    tempRect.setBounds(getPaintRectangle(figure, insets));
    Rectangle r = tempRect;
    if (r.isEmpty())
      return;

    g.setLineStyle(linestyle);
    if (bold)
      g.setLineWidth(3);
    else
      g.setLineWidth(1);
    Rectangle textLoc = new Rectangle(r.getTopLeft(), getTextExtents(figure));
    textLoc.y += STD_MARGIN;
    textLoc.height += STD_MARGIN * 2;
    r.width--;
    r.height--;
    g.drawRectangle(r);

    if (getUnderline())
    {
      r.height = textLoc.height;
      g.drawRectangle(r);
    }

    textLoc.x = r.x + (r.width - textLoc.width) / 2;
    textLoc.y += (r.height - textLoc.height) / 2;
    g.setFont(getFont(figure));
    g.setForegroundColor(getTextColor());
    g.clipRect(textLoc);
    g.fillText(getLabel(), textLoc.getTopLeft());
    if (icon != null)
    {
      g.drawImage(icon, 
                  r.x + r.width - icon.getBounds().width - STD_MARGIN, 
                  r.y + (r.height - icon.getBounds().height) / 2);
    }
  }

  /**
   * @see org.eclipse.draw2d.Border#getPreferredSize(IFigure)
   */
  public Dimension getPreferredSize(IFigure fig) 
  {
    Dimension size = getTextExtents(fig).getCopy();
    size.expand(size.height, 0);
    if (getIcon() != null)
    {
      size.expand((getIcon().getBounds().width + STD_MARGIN) * 2, 0);
      size.union(new Dimension(0, getIcon().getBounds().height + STD_MARGIN * 2));      
    }
    return size;
  }

  /**
   * Gets the underline attribute.
   * @return the underline attribute.
   */
  public boolean getUnderline()
  {
    return underline;
  }

  /**
   * Sets the underline attribute.
   * @param underline - the underline attribute.
   */
  public void setUnderline(boolean underline)
  {
    this.underline = underline;
  }

  /**
   * Gets the line style
   * @return the line style
   * @see org.eclipse.draw2d.Graphics
   */
  public int getLinestyle()
  {
    return linestyle;
  }

  /**
   * Gets the line style
   * @param linestyle the line style
   * @see org.eclipse.draw2d.Graphics
   */
  public void setLinestyle(int linestyle)
  {
    this.linestyle = linestyle;
  }

  /**
   * Sets the icon image.
   * @param icon the icon.
   */
  public void setIcon(Image icon)
  {
    this.icon = icon;
  }

  /**
   * Gets the icon image.
   * @return the icon.
   */
  public Image getIcon()
  {
    return icon;
  }

  /**
   * Returns true if the border is bold.
   * @return true if the border is bold.
   */
  public boolean isBold()
  {
    return bold;
  }
  
  /**
   * Returns true if the border is bold.
   * @param bold true if the border is bold.
   */
  public void setBold(boolean bold)
  {
    this.bold = bold;
  }
}
