/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose.eclipse.log;

import java.util.Enumeration;
import java.util.Stack;

import org.eclipse.core.runtime.ILog;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.swt.widgets.Shell;

import be.ac.vub.cocompose.log.Log;
import be.ac.vub.cocompose.eclipse.CoComposePlugin;

/**
 * Core Eclipse debugging functionality.
 * @author Dennis Wagelaar
 * @version 0.2.8, 19/04/2004
 * @since 0.2.7
 */
public class EclipseLog implements Log
{
  private Stack lastLogs = new Stack();
  private String uid = 
    CoComposePlugin.getDefault().getDescriptor().getUniqueIdentifier();
  private ILog log = 
    CoComposePlugin.getDefault().getLog();
  private Shell shell =
    CoComposePlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell();
  
  /**
   * @see be.ac.vub.cocompose.log.Log#log(java.lang.String, int, java.lang.Throwable)
   */
  public void log(String message, int level, Throwable exception)
  {
    if (!isDebugging() && level < WARNING)
      return;
  	Enumeration levels = lastLogs.elements();
  	while (levels.hasMoreElements())
  	{
      levels.nextElement();
      message = "    " + message;
  	}
    IStatus st = new Status(level, uid, IStatus.OK, message, exception);
    if (lastLogs.size() == 1)
    {
      IStatus root = (IStatus) lastLogs.pop();
      if (!(root instanceof MultiStatus))
      {
        root = new MultiStatus(
          root.getPlugin(), root.getCode(), root.getMessage(), root.getException());
      }
      lastLogs.push(root);
    }
    if (!lastLogs.isEmpty())
      ((MultiStatus) lastLogs.get(0)).add(st);
    lastLogs.push(st);
  }
  
  /**
   * @see be.ac.vub.cocompose.log.Log#endLog()
   */
  public void endLog(int level)
  {
    if (!isDebugging() && level < WARNING)
      return;
    if (!lastLogs.isEmpty())
    {
      IStatus st = (IStatus) lastLogs.pop();
      if (lastLogs.isEmpty())
        log.log(st);
    }
  }

  /**
   * Reports an exception/error in the eclipse log.
   * @param e the exception to log.
   */
  public void report(Throwable e)
  {
    log(e.getMessage(), ERROR, e);
    IStatus st = (IStatus) lastLogs.peek();
    flushLog();
    ErrorDialog dlg = new ErrorDialog(
      shell,
      "Error",
      st.getMessage(),
      st,
      IStatus.ERROR);
    dlg.open();
  }

  /**
   * Returns true if debugging messages (OK, INFO) should be logged.
   * @return true if debugging messages should be logged.
   */
  public boolean isDebugging()
  {
    return CoComposePlugin.getDefault().isDebugging();
  }

  /**
   * Sets debugging message logging (OK, INFO).
   * @param b if true, log debug messages.
   */
  public void setDebugging(boolean b)
  {
    CoComposePlugin.getDefault().setDebugging(b);
  }
  
  /**
   * Flushes all logs still in the stack.
   */
  private void flushLog()
  {
    while (!lastLogs.isEmpty())
      endLog(ERROR);
  }
}
