/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose.io.xmi;

import java.util.Iterator;
import org.xml.sax.SAXException;
import org.xml.sax.ContentHandler;
import org.xml.sax.Attributes;
import org.xml.sax.helpers.AttributesImpl;
import be.ac.vub.cocompose.lang.core.Model;
import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.core.ImplementationGenerator;
import be.ac.vub.cocompose.lang.core.ImplementationRolePart;
import be.ac.vub.cocompose.lang.core.ModelElement;
import be.ac.vub.cocompose.lang.core.Namespace;
import be.ac.vub.cocompose.lang.core.Refinement;
import be.ac.vub.cocompose.lang.core.Relationship;
import be.ac.vub.cocompose.lang.core.RelationElement;
import be.ac.vub.cocompose.lang.core.SolutionPattern;
import be.ac.vub.cocompose.lang.core.ImplementationPattern;
import be.ac.vub.cocompose.lang.core.RefinedElement;
import be.ac.vub.cocompose.lang.core.Concept;
import be.ac.vub.cocompose.lang.core.Role;
import be.ac.vub.cocompose.lang.constraints.CompositeConstraint;
import be.ac.vub.cocompose.lang.constraints.Constraint;
import be.ac.vub.cocompose.lang.constraints.ElementConstraint;
import be.ac.vub.cocompose.lang.properties.ModelProperty;
import be.ac.vub.cocompose.lang.DefaultSuperModelVisitor;
import be.ac.vub.cocompose.lang.ModelVisitor;
import be.ac.vub.cocompose.log.Log;

/**
 * Writes a CoCompose model to an XMI file.
 * @author Dennis Wagelaar 
 * @version 0.2.0, 09/01/2004
 * @since 0.2.0
 */
public class XMIElementWriter extends DefaultSuperModelVisitor 
{
  private ContentHandler xmlHandler = null;
  private XMIElementAttributesWriter attributes =
    new XMIElementAttributesWriter();

  /**
   * Sets the XML handler.
   * @param xmlHandler - the XML handler.
   */
  public void setXmlHandler(ContentHandler xmlHandler)
  {
    this.xmlHandler = xmlHandler;
  }

  /**
   * Gets the XML handler.
   * @return the XML handler.
   */
  public ContentHandler getXmlHandler()
  {
    return xmlHandler;
  }

  /**
   * Visits a Namespace.
   * @param namespace - the element to visit.
   * @exception ModelElementException
   */
  public void visitNamespace(Namespace namespace) throws ModelElementException
  {
    try
    {
      current.visitModelElement(namespace);
      Iterator ownedElts = namespace.getOwnedElements().iterator();
      while (ownedElts.hasNext())
      {
        ModelElement elt = (ModelElement) ownedElts.next();
        attributes.visit(elt);
        xmlHandler.startElement(XMI.CoCompose.NS_URI, "ownedElement", 
          XMI.CoCompose.qName("ownedElement"), attributes.getAttributes());
        elt.accept(current);
        xmlHandler.endElement(XMI.CoCompose.NS_URI, "ownedElement", 
          XMI.CoCompose.qName("ownedElement"));
      }
    }
    catch (SAXException e)
    {
      throw new ModelElementException(e, namespace, Log.ERROR);
    }
  }

  /**
   * Visits a Model.
   * @param model - the element to visit.
   * @exception ModelElementException
   */
  public void visitModel(Model model) throws ModelElementException
  {
    try
    {
      attributes.visit(model);
      xmlHandler.startElement(XMI.CoCompose.NS_URI, "Model", 
        XMI.CoCompose.qName("Model"), attributes.getAttributes());
      super.visitModel(model);
      xmlHandler.endElement(XMI.CoCompose.NS_URI, "Model", 
        XMI.CoCompose.qName("Model"));
    }
    catch (SAXException e)
    {
      throw new ModelElementException(e, model, Log.ERROR);
    }
  }

  /**
   * Visits a Relationship.
   * @param relationship - the element to visit.
   * @exception ModelElementException
   */
  public void visitRelationship(Relationship relationship) throws ModelElementException
  {
    try
    {
      super.visitRelationship(relationship);
      AttributesImpl atts = new AttributesImpl();
      // first participant
      atts.addAttribute(XMI.NS_URI, "idref", XMI.qName("idref"), "IDREF", relationship.getSourceParticipant().getId());
      xmlHandler.startElement(XMI.CoCompose.NS_URI, "participant", XMI.CoCompose.qName("participant"), atts);
      xmlHandler.endElement(XMI.CoCompose.NS_URI, "participant", XMI.CoCompose.qName("participant"));
      // second participant
      atts.clear();
      atts.addAttribute(XMI.NS_URI, "idref", XMI.qName("idref"), "IDREF", relationship.getTargetParticipant().getId());
      xmlHandler.startElement(XMI.CoCompose.NS_URI, "participant", XMI.CoCompose.qName("participant"), atts);
      xmlHandler.endElement(XMI.CoCompose.NS_URI, "participant", XMI.CoCompose.qName("participant"));
    }
    catch (SAXException e)
    {
      throw new ModelElementException(e, relationship, Log.ERROR);
    }
  }

  /**
   * Visits a RelationElement.
   * @param relationElement - the element to visit.
   * @exception ModelElementException
   */
  public void visitRelationElement(RelationElement relationElement) throws ModelElementException
  {
    try
    {
      super.visitRelationElement(relationElement);
      AttributesImpl atts = new AttributesImpl();
      Iterator relations = relationElement.getRelationships().iterator();
      while (relations.hasNext())
      {
        Relationship relation = (Relationship) relations.next();
        atts.clear();
        atts.addAttribute(XMI.NS_URI, "idref", XMI.qName("idref"), "IDREF", relation.getId());
        xmlHandler.startElement(XMI.CoCompose.NS_URI, "relationship", XMI.CoCompose.qName("relationship"), atts);
        xmlHandler.endElement(XMI.CoCompose.NS_URI, "relationship", XMI.CoCompose.qName("relationship"));
      }
      // xmi:Extension
      xmlHandler.startElement(XMI.NS_URI, "Extension", XMI.qName("Extension"), null);
      // Diagram:GraphicElement
      xmlHandler.startElement(XMI.Diagram.NS_URI, "GraphicElement", XMI.Diagram.qName("GraphicElement"), null);
      // position
      atts.clear();
      atts.addAttribute(XMI.Diagram.NS_URI, "x", XMI.Diagram.qName("x"), "CDATA", new Integer(relationElement.getPosition().x).toString());
      atts.addAttribute(XMI.Diagram.NS_URI, "y", XMI.Diagram.qName("y"), "CDATA", new Integer(relationElement.getPosition().y).toString());
      xmlHandler.startElement(XMI.Diagram.NS_URI, "position", XMI.Diagram.qName("position"), atts);
      xmlHandler.endElement(XMI.Diagram.NS_URI, "position", XMI.Diagram.qName("position"));
      // color
      atts.clear();
      atts.addAttribute(XMI.Diagram.NS_URI, "r", XMI.Diagram.qName("r"), "CDATA", new Integer(relationElement.getBackgroundColor().getRed()).toString());
      atts.addAttribute(XMI.Diagram.NS_URI, "g", XMI.Diagram.qName("g"), "CDATA", new Integer(relationElement.getBackgroundColor().getGreen()).toString());
      atts.addAttribute(XMI.Diagram.NS_URI, "b", XMI.Diagram.qName("b"), "CDATA", new Integer(relationElement.getBackgroundColor().getBlue()).toString());
      xmlHandler.startElement(XMI.Diagram.NS_URI, "color", XMI.Diagram.qName("color"), atts);
      xmlHandler.endElement(XMI.Diagram.NS_URI, "color", XMI.Diagram.qName("color"));
      // close elements
      xmlHandler.endElement(XMI.Diagram.NS_URI, "GraphicElement", XMI.Diagram.qName("GraphicElement"));
      xmlHandler.endElement(XMI.NS_URI, "Extension", XMI.qName("Extension"));
    }
    catch (SAXException e)
    {
      throw new ModelElementException(e, relationElement, Log.ERROR);
    }
  }

  /**
   * Visits a SolutionPattern.
   * @param solutionPattern - the element to visit.
   * @exception ModelElementException
   */
  public void visitSolutionPattern(SolutionPattern solutionPattern) throws ModelElementException
  {
    try
    {
      super.visitSolutionPattern(solutionPattern);
      if (solutionPattern.getModelUri() != null)
      {
        AttributesImpl atts = new AttributesImpl();
        atts.addAttribute(XMI.NS_URI, "href", XMI.qName("href"), "CDATA",
          solutionPattern.getModelUri());
        xmlHandler.startElement(XMI.CoCompose.NS_URI, "modelUri", XMI.CoCompose.qName("modelUri"), atts);
        xmlHandler.endElement(XMI.CoCompose.NS_URI, "modelUri", XMI.CoCompose.qName("modelUri"));
      }
    }
    catch (SAXException e)
    {
      throw new ModelElementException(e, solutionPattern, Log.ERROR);
    }
  }

  /**
   * Visits an ImplementationGenerator.
   * @param implementationGenerator - the element to visit.
   * @exception ModelElementException
   */
  public void visitImplementationGenerator(ImplementationGenerator implementationGenerator) throws ModelElementException
  {
    try
    {
      super.visitImplementationGenerator(implementationGenerator);
      AttributesImpl atts = new AttributesImpl();
      Iterator roleparts = implementationGenerator.getRoleParts().iterator();
      while (roleparts.hasNext())
      {
        ImplementationRolePart rp = (ImplementationRolePart) roleparts.next();
        atts.clear();
        atts.addAttribute(XMI.NS_URI, "idref", XMI.qName("idref"), "IDREF", rp.getId());
        xmlHandler.startElement(XMI.CoCompose.NS_URI, "rolePart", XMI.CoCompose.qName("rolePart"), atts);
        xmlHandler.endElement(XMI.CoCompose.NS_URI, "rolePart", XMI.CoCompose.qName("rolePart"));
      }
    }
    catch (SAXException e)
    {
      throw new ModelElementException(e, implementationGenerator, Log.ERROR);
    }
  }

  /**
   * Visits an ImplementationPattern.
   * @param implementationPattern - the element to visit.
   * @exception ModelElementException
   */
  public void visitImplementationPattern(ImplementationPattern implementationPattern) throws ModelElementException
  {
    try
    {
      super.visitImplementationPattern(implementationPattern);
      if (implementationPattern.getContent() != null)
      {
        xmlHandler.startElement(XMI.CoCompose.NS_URI, "content", XMI.CoCompose.qName("content"), null);
        xmlHandler.characters(implementationPattern.getContent().toCharArray(), 0,
          implementationPattern.getContent().length());
        xmlHandler.endElement(XMI.CoCompose.NS_URI, "content", XMI.CoCompose.qName("content"));
      }
    }
    catch (SAXException e)
    {
      throw new ModelElementException(e, implementationPattern, Log.ERROR);
    }
  }

  /**
   * Visits a RefinedElement.
   * @param refinedElement - the element to visit.
   * @exception ModelElementException
   */
  public void visitRefinedElement(RefinedElement refinedElement) throws ModelElementException
  {
    try
    {
      super.visitRefinedElement(refinedElement);
      AttributesImpl atts = new AttributesImpl();
      Iterator refinements = refinedElement.getRefinements().iterator();
      while (refinements.hasNext())
      {
        Refinement ref = (Refinement) refinements.next();
        atts.clear();
        atts.addAttribute(XMI.NS_URI, "idref", XMI.qName("idref"), "IDREF", ref.getId());
        xmlHandler.startElement(XMI.CoCompose.NS_URI, "refinement", XMI.CoCompose.qName("refinement"), atts);
        xmlHandler.endElement(XMI.CoCompose.NS_URI, "refinement", XMI.CoCompose.qName("refinement"));
      }
      atts = new AttributesImpl();
      Iterator inhElts = refinedElement.getInheritsFrom().iterator();
      while (inhElts.hasNext())
      {
        RefinedElement elt = (RefinedElement) inhElts.next();
        atts.clear();
        atts.addAttribute(XMI.NS_URI, "idref", XMI.qName("idref"), "IDREF", elt.getId());
        xmlHandler.startElement(XMI.CoCompose.NS_URI, "inheritsFrom", XMI.CoCompose.qName("inheritsFrom"), atts);
        xmlHandler.endElement(XMI.CoCompose.NS_URI, "inheritsFrom", XMI.CoCompose.qName("inheritsFrom"));
      }
      atts = new AttributesImpl();
      Iterator superElts = refinedElement.getSuperimposesOn().iterator();
      while (superElts.hasNext())
      {
        RefinedElement elt = (RefinedElement) superElts.next();
        atts.clear();
        atts.addAttribute(XMI.NS_URI, "idref", XMI.qName("idref"), "CDATA", elt.getId());
        xmlHandler.startElement(XMI.CoCompose.NS_URI, "superimposesOn", XMI.CoCompose.qName("superimposesOn"), atts);
        xmlHandler.endElement(XMI.CoCompose.NS_URI, "superimposesOn", XMI.CoCompose.qName("superimposesOn"));
      }
    }
    catch (SAXException e)
    {
      throw new ModelElementException(e, refinedElement, Log.ERROR);
    }
  }

  /**
   * Visits a concept.
   * @param concept - the element to visit.
   * @exception ModelElementException
   */
  public void visitConcept(Concept concept) throws ModelElementException
  {
    try
    {
      super.visitConcept(concept);
      AttributesImpl atts = new AttributesImpl();
      Iterator partOf = concept.getPartOf().iterator();
      while (partOf.hasNext())
      {
        Role role = (Role) partOf.next();
        atts.clear();
        atts.addAttribute(XMI.NS_URI, "idref", XMI.qName("idref"), "CDATA", role.getId());
        xmlHandler.startElement(XMI.CoCompose.NS_URI, "partOf", XMI.CoCompose.qName("partOf"), atts);
        xmlHandler.endElement(XMI.CoCompose.NS_URI, "partOf", XMI.CoCompose.qName("partOf"));
      }
    }
    catch (SAXException e)
    {
      throw new ModelElementException(e, concept, Log.ERROR);
    }
  }

  /**
   * Visits a Role.
   * @param role - the element to visit.
   * @exception ModelElementException
   */
  public void visitRole(Role role) throws ModelElementException
  {
    try
    {
      super.visitRole(role);
      attributes.visit(role.getMultiplicity());
      xmlHandler.startElement(XMI.CoCompose.NS_URI, "multiplicity", XMI.CoCompose.qName("multiplicity"), attributes.getAttributes());
      role.getMultiplicity().accept(current);
      xmlHandler.endElement(XMI.CoCompose.NS_URI, "multiplicity", XMI.CoCompose.qName("multiplicity"));
      AttributesImpl atts = new AttributesImpl();
      Iterator parts = role.getParts().iterator();
      while (parts.hasNext())
      {
        Concept concept = (Concept) parts.next();
        atts.clear();
        atts.addAttribute(XMI.NS_URI, "idref", XMI.qName("idref"), "CDATA", concept.getId());
        xmlHandler.startElement(XMI.CoCompose.NS_URI, "part", XMI.CoCompose.qName("part"), atts);
        xmlHandler.endElement(XMI.CoCompose.NS_URI, "part", XMI.CoCompose.qName("part"));
      }
    }
    catch (SAXException e)
    {
      throw new ModelElementException(e, role, Log.ERROR);
    }
  }

  /**
   * Visits an ElementConstraint.
   * @param elementConstraint - the element to visit.
   * @exception ModelElementException
   */
  public void visitElementConstraint(ElementConstraint elementConstraint) throws ModelElementException
  {
    try
    {
      super.visitElementConstraint(elementConstraint);
      AttributesImpl atts = new AttributesImpl();
      Iterator constrElts = elementConstraint.getConstrainedElements().iterator();
      while (constrElts.hasNext())
      {
        ModelElement constrElt = (ModelElement) constrElts.next();
        atts.clear();
        atts.addAttribute(XMI.NS_URI, "idref", XMI.qName("idref"), "IDREF", constrElt.getId());
        xmlHandler.startElement(XMI.CoCompose.NS_URI, "constrainedElement", XMI.CoCompose.qName("constrainedElement"), atts);
        xmlHandler.endElement(XMI.CoCompose.NS_URI, "constrainedElement", XMI.CoCompose.qName("constrainedElement"));
      }
    }
    catch (SAXException e)
    {
      throw new ModelElementException(e, elementConstraint, Log.ERROR);
    }
  }

  /**
   * Visits a CompositeConstraint.
   * @param compositeConstraint - the element to visit.
   * @exception ModelElementException
   */
  public void visitCompositeConstraint(CompositeConstraint compositeConstraint) throws ModelElementException
  {
    try
    {
      super.visitCompositeConstraint(compositeConstraint);
      AttributesImpl atts = new AttributesImpl();
      Iterator constr = compositeConstraint.getOwnedConstraints().iterator();
      while (constr.hasNext())
      {
        Constraint c = (Constraint) constr.next();
        atts.clear();
        atts.addAttribute(XMI.NS_URI, "idref", XMI.qName("idref"), "CDATA", c.getId());
        xmlHandler.startElement(XMI.CoCompose.NS_URI, "ownedConstraint", XMI.CoCompose.qName("ownedConstraint"), atts);
        xmlHandler.endElement(XMI.CoCompose.NS_URI, "ownedConstraint", XMI.CoCompose.qName("ownedConstraint"));
      }
    }
    catch (SAXException e)
    {
      throw new ModelElementException(e, compositeConstraint, Log.ERROR);
    }
  }

  /**
   * Visits a ModelProperty.
   * @param modelProperty - the element to visit.
   * @exception ModelElementException
   */
  public void visitModelProperty(ModelProperty modelProperty) throws ModelElementException
  {
    try
    {
      super.visitModelProperty(modelProperty);
      if (modelProperty.getModel() != null)
      {
        AttributesImpl atts = new AttributesImpl();
        atts.addAttribute(XMI.NS_URI, "href", XMI.qName("href"), "CDATA",
          modelProperty.getModel().getUri() + '#' +
          modelProperty.getModel().getId());
        xmlHandler.startElement(XMI.CoCompose.NS_URI, "model", XMI.CoCompose.qName("model"), atts);
        xmlHandler.endElement(XMI.CoCompose.NS_URI, "model", XMI.CoCompose.qName("model"));
      }
    }
    catch (SAXException e)
    {
      throw new ModelElementException(e, modelProperty, Log.ERROR);
    }
  }

  /**
   * Gets the root attributes to add to the first element.
   * @return the root attributes.
   */
  public Attributes getRootAttributes()
  {
    return attributes.getRootAttributes();
  }

  /**
   * Sets the root attributes to add to the first element.
   * @param rootAttributes - the root attributes.
   */
  public void setRootAttributes(Attributes rootAttributes)
  {
    attributes.setRootAttributes(rootAttributes);
  }

  /**
   * Sets a stacked visitor for this and the contained visitors.
   * @param stacked - the stacked visitor.
   */
  public void setStacked(ModelVisitor stacked)
  {
    super.setStacked(stacked);
    attributes.setStacked(stacked);
  }
}
