/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: XMIModelReader.java,v 1.5 2004/04/28 12:23:29 dwagelaa Exp $
 * $Log: XMIModelReader.java,v $
 * Revision 1.5  2004/04/28 12:23:29  dwagelaa
 * + added parsing of ImplementationPattern.content tag
 *
 */

package be.ac.vub.cocompose.io.xmi;

import java.awt.Color;
import java.awt.Point;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Hashtable;

import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

import be.ac.vub.cocompose.io.ModelReader;
import be.ac.vub.cocompose.io.xml.XML;
import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.ModelFactory;
import be.ac.vub.cocompose.lang.core.Model;
import be.ac.vub.cocompose.lang.core.ModelElement;
import be.ac.vub.cocompose.lang.core.Namespace;
import be.ac.vub.cocompose.lang.core.RelationElement;
import be.ac.vub.cocompose.lang.core.Relationship;
import be.ac.vub.cocompose.lang.core.Role;
import be.ac.vub.cocompose.lang.datatypes.Multiplicity;

/** 
 * Reads a CoCompose model from an XMI file (requires second pass by XMIModelLinkReader). 
 * @author Dennis Wagelaar 
 * @version 0.2.9, 28/04/2004
 * @since 0.2.0
 */
public class XMIModelReader extends DefaultHandler implements ModelReader 
{
  private Model model = null;
  private Hashtable elementMap = new Hashtable();
  private XMIElementAttributesReader elementReader = null;
  private XMIElementTypeSwitch typeSwitch = null;
  private XMIModelLinkReader linkReader = null;
  private XMLReader xmlParser = null;
  private Namespace namespace = null;
  private ModelElement element = null;
  private Relationship relationship = null;

  /**
   * Reads the model from its uri.
   * @param uri - the uri to read from.
   * @param factory - the model factory used to create the elements.
   * @return the model
   * @exception IOException - if the model could not be read.
   * @exception ModelElementException - if a model element could not be read.
   */
  public Model read(String uri, ModelFactory factory) throws IOException, ModelElementException
  {
    uri = uri.replace('\\', '/');
    try
    {
      preRead(factory);
      // start parser for 1st pass
      getXmlParser().setContentHandler(this);
      getXmlParser().parse(uri);
      // start parser for 2nd pass
      getXmlParser().setContentHandler(getLinkReader());
      getXmlParser().parse(uri);
      return getModel();
    }
    catch (Exception e)
    {
      throw new ModelElementException(e);
    }
  }

  /**
   * Reads the model from its uri.
   * @param input - the stream to read from.
   * @param factory - the model factory used to create the elements.
   * @return the model
   * @exception IOException - if the model could not be read.
   * @exception ModelElementException - if a model element could not be read.
   */
  public Model read(InputStream input, ModelFactory factory) throws IOException, ModelElementException
  {
    byte[] contents = convertInputStream(input);
    try
    {
      preRead(factory);
      // start parser for 1st pass
      InputSource source = new InputSource(new ByteArrayInputStream(contents));
      getXmlParser().setContentHandler(this);
      getXmlParser().parse(source);
      // start parser for 2nd pass
      source = new InputSource(new ByteArrayInputStream(contents));
      getXmlParser().setContentHandler(getLinkReader());
      getXmlParser().parse(source);
      return getModel();
    }
    catch (Exception e)
    {
      throw new ModelElementException(e);
    }
    finally
    {
      input.close();
    }
  }
  
  /**
   * Prepares for read().
   * @param factory - the factory to create the model and the elements.
   * @throws Exception if pre-read setup failed.
   */
  protected void preRead(ModelFactory factory) throws Exception
  {
    setModel(factory.createModel());
    setNamespace(getModel());
    setElementReader(new XMIElementAttributesReader());
    setTypeSwitch(new XMIElementTypeSwitch(factory));
    setLinkReader(new XMIModelLinkReader(getModel(), getElementMap()));
    setXmlParser((XMLReader) Class.forName(XML.PARSER).newInstance());
    getXmlParser().setFeature(XML.VALIDATION, XML.SetValidation);
    getXmlParser().setFeature(XML.NAMESPACES, XML.SetNamespaces);
    getXmlParser().setFeature(XML.SCHEMA, XML.SetSchema);
    getXmlParser().setFeature(XML.SCHEMA_FULL, XML.SetSchemaFull);
  }
  
  /**
   * Converts the given InputStream to a byte array, so that it
   * can be read more than once. The InputStream will be closed
   * after converting.
   * @param input the InputStream to convert.
   * @return the byte array copy of the InputStream.
   * @throws IOException if the stream cannot be converted.
   */
  private byte[] convertInputStream(InputStream input) throws IOException
  {
    byte[] buffer = new byte[input.available()];
    input.read(buffer);
    input.close();
    return buffer;
  }

  //
  // ContentHandler methods
  //

  /**
   * Processing instruction.
   *
   * @param target Processing instruction element name.
   * @param data Processing instruction element data.
   */
  public void processingInstruction(String target, String data)
    throws SAXException
  {
  } // processingInstruction

  /**
   * Start document.
   */
  public void startDocument() throws SAXException
  {
  } //startDocument

  /**
   * Start element.
   * @param uri XML namespace URI.
   * @param local local element name.
   * @param raw raw (qualified) element name.
   * @param attrs element attributes.
   * @exception SAXException
   */
  public void startElement(String uri, String local, String raw,
    Attributes attrs) throws SAXException
  {
    if (uri == null)
      throw new SAXException("XML namespace URI may not be null");
    if (uri.equals(XMI.NS_URI))
      startXMIElement(local, attrs);
    else
      startCoComposeElement(local, attrs);
  } //startElement

  /**
   * Start XMI element.
   * @param local local element name.
   * @param attrs element attributes.
   * @exception SAXException
   */
  public void startXMIElement(String local, Attributes attrs) throws SAXException
  {
  }

  /**
   * Start CoCompose element.
   * @param local local element name.
   * @param attrs element attributes.
   * @exception SAXException
   */
  public void startCoComposeElement(String local, Attributes attrs) throws SAXException
  {
    try
    {
      if (local.equals("Model"))
      {
        getElementReader().setAttributes(attrs);
        getElementReader().visit(getModel());
        getElementMap().put(getModel().getId(), getModel());
      }
      else if (local.equals("ownedElement"))
      {
        startOwnedElement(attrs);
      }
      else if (local.equals("multiplicity"))
      {
        startMultiplicity(attrs);
      }
      else if (local.equals("position"))
      {
        startPosition(attrs);
      }
      else if (local.equals("color"))
      {
        startColor(attrs);
      }
    }
    catch (ModelElementException e)
    {
      SAXException se = new SAXException(e);
      se.setStackTrace(e.getStackTrace());
      throw se;
    }
  }

  /**
   * Parses an ownedElement tag
   * @param attrs element attributes
   * @throws ModelElementException
   */
  public void startOwnedElement(Attributes attrs) throws ModelElementException
  {
    String type = attrs.getValue(XMI.qName("type"));
    setElement(getTypeSwitch().createElement(type));
    if (getElement() == null)
      throw new ModelElementException("element type " + type + " not recognized");
    getElementReader().setAttributes(attrs);
    getElementReader().visit(getElement());
    if (getNamespace() == null)
      throw new ModelElementException("Namespace is null for " + type + " " + getElement().getName());
    getNamespace().addOwnedElement(getElement());
    getElementMap().put(getElement().getId(), getElement());
    if (getElement() instanceof Namespace)
      setNamespace((Namespace) getElement());
    else if (getElement() instanceof Relationship)
      setRelationship((Relationship) getElement());
  }

  /**
   * Parses a multiplicity tag
   * @param attrs element attributes
   * @throws ModelElementException
   */
  public void startMultiplicity(Attributes attrs) throws ModelElementException
  {
    if (getNamespace() instanceof Role)
    {
      try
      {
        Multiplicity mult = new Multiplicity();
        mult.setLower(Integer.parseInt(attrs.getValue("lower")));
        mult.setUpper(Integer.parseInt(attrs.getValue("upper")));
        ((Role) getNamespace()).setMultiplicity(mult);
      }
      catch (Exception e)
      {
        throw new ModelElementException(e);
      }
    }
    else
    {
      throw new ModelElementException("Cannot set multiplicity for " + 
        getNamespace(), getNamespace());
    }
  }

  /**
   * Finalises an ownedElement tag
   * @throws ModelElementException
   */
  public void endOwnedElement() throws ModelElementException
  {
    if ((getElement() instanceof Namespace) && (getNamespace() != null))
      setNamespace(getNamespace().getNamespace());
  }

  /**
   * Parses a position tag
   * @param attrs element attributes
   * @throws ModelElementException
   */
  public void startPosition(Attributes attrs) throws ModelElementException
  {
    if (!(getNamespace() instanceof RelationElement))
      throw new ModelElementException("Cannot find relation element to set position for");
    RelationElement element = (RelationElement) getNamespace();
    try
    {
      Point pos = new Point();
      pos.x = Integer.parseInt(attrs.getValue(XMI.Diagram.qName("x")));
      pos.y = Integer.parseInt(attrs.getValue(XMI.Diagram.qName("y")));
      element.setPosition(pos);
    }
    catch (NumberFormatException e)
    {
      throw new ModelElementException(e);
    }
  }

  /**
   * Parses a color tag
   * @param attrs element attributes
   * @throws ModelElementException
   */
  public void startColor(Attributes attrs) throws ModelElementException
  {
    if (!(getNamespace() instanceof RelationElement))
      throw new ModelElementException("Cannot find relation element to set color for");
    RelationElement element = (RelationElement) getNamespace();
    try
    {
      Color col = new Color(
        Integer.parseInt(attrs.getValue(XMI.Diagram.qName("r"))),
        Integer.parseInt(attrs.getValue(XMI.Diagram.qName("g"))),
        Integer.parseInt(attrs.getValue(XMI.Diagram.qName("b"))));
      element.setBackgroundColor(col);
    }
    catch (NumberFormatException e)
    {
      throw new ModelElementException(e);
    }
  }
  
  /**
   * Characters.
   *
   * @param ch Characters to print.
   * @param start Starting index of characters.
   * @param length Length of characters.
   */
  public void characters(char[] ch, int start, int length)
    throws SAXException
  {
  } //characters

  /**
   * Ignorable whitespace.
   *
   * @param ch Characters to print.
   * @param start Starting index of characters.
   * @param length Length of characters.
   */
  public void ignorableWhitespace(char[] ch, int start, int length)
    throws SAXException
  {
  } //ignorableWhitespace

  /**
   * End element.
   *
   * @param uri Ignored.
   * @param local Ignored.
   * @param raw Element name.
   */
  public void endElement(String uri, String local, String raw)
    throws SAXException
  {
    if (uri == null)
      throw new SAXException("XML namespace URI may not be null");
    if (uri.equals(XMI.NS_URI))
      endXMIElement(local);
    else
      endCoComposeElement(local);
  } //endElement

  /**
   * End XMI element.
   * @param local local element name.
   * @exception SAXException
   */
  public void endXMIElement(String local) throws SAXException
  {
  }

  /**
   * Start CoCompose element.
   * @param local local element name.
   * @exception SAXException
   */
  public void endCoComposeElement(String local) throws SAXException
  {
    try
    {
      if (local.equals("ownedElement"))
      {
        endOwnedElement();
      }
    }
    catch (ModelElementException e)
    {
      throw new SAXException(e.getMessage());
    }
  }

  //
  // DTDHandler methods
  //

  /**
   * DTD declaration.
   *
   * @param name DTD name.
   * @param publicId DTD public ID.
   * @param systemId DTD system ID.
   */
  public void notationDecl(String name, String publicId, String systemId)
    throws SAXException
  {
  } //notationDecl


  protected void setElement(ModelElement element)
  {
    this.element = element;
  }

  public ModelElement getElement()
  {
    return element;
  }

  protected void setElementMap(Hashtable elementMap)
  {
    this.elementMap = elementMap;
  }

  public Hashtable getElementMap()
  {
    return elementMap;
  }

  protected void setElementReader(XMIElementAttributesReader elementReader)
  {
    this.elementReader = elementReader;
  }

  public XMIElementAttributesReader getElementReader()
  {
    return elementReader;
  }

  protected void setLinkReader(XMIModelLinkReader linkReader)
  {
    this.linkReader = linkReader;
  }

  public XMIModelLinkReader getLinkReader()
  {
    return linkReader;
  }

  protected void setModel(Model model)
  {
    this.model = model;
  }

  public Model getModel()
  {
    return model;
  }

  protected void setNamespace(Namespace namespace)
  {
    this.namespace = namespace;
  }

  public Namespace getNamespace()
  {
    return namespace;
  }

  protected void setRelationship(Relationship relationship)
  {
    this.relationship = relationship;
  }

  public Relationship getRelationship()
  {
    return relationship;
  }

  protected void setTypeSwitch(XMIElementTypeSwitch typeSwitch)
  {
    this.typeSwitch = typeSwitch;
  }

  public XMIElementTypeSwitch getTypeSwitch()
  {
    return typeSwitch;
  }

  protected void setXmlParser(XMLReader xmlParser)
  {
    this.xmlParser = xmlParser;
  }

  public XMLReader getXmlParser()
  {
    return xmlParser;
  }
}
