/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003 - 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */

package be.ac.vub.cocompose.lang.impl.constraints;

import java.util.Collection;
import java.util.Vector;
import java.util.Iterator;
import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.constraints.ElementConstraint;
import be.ac.vub.cocompose.lang.core.ModelElement;
import be.ac.vub.cocompose.lang.core.Namespace;
import be.ac.vub.cocompose.lang.ModelVisitor;
import be.ac.vub.cocompose.log.Log;

/**
 * Model element constraint.
 * @author Dennis Wagelaar 
 * @version 0.2.8, 19/04/2004
 * @since 0.2.0
 */
public class DefaultElementConstraint extends DefaultConstraint implements ElementConstraint
{
  private Vector constrainedElements = new Vector();

  /**
   * Returns the constrained elements.
   * @return a copy of the collection of constrained elements.
   */
  public Collection getConstrainedElements()
  {
    return (Collection) constrainedElements.clone();
  }

  /**
   * Accepts a visitor.
   * @param modelVisitor The visitor to accept.*/
  public void accept(ModelVisitor modelVisitor) throws ModelElementException 
  {
    modelVisitor.visitElementConstraint(this);
  }

  /**
   * Invokes superclass accept().
   * @param modelVisitor The visitor to accept.
   * @see #accept(ModelVisitor)*/
  public void acceptSuper(ModelVisitor modelVisitor) throws ModelElementException
  {
    super.accept(modelVisitor);
  }

  /**
   * Adds a constrained element.
   * @param constrainedElement - the element to add. 
   */
  public void addConstrainedElement(ModelElement constrainedElement)
  {
    constrainedElements.add(constrainedElement);
  }

  /**
   * Removes a constrained element.
   * @param constrainedElement - the element to remove. 
   */
  public void removeConstrainedElement(ModelElement constrainedElement)
  {
    constrainedElements.remove(constrainedElement);
  }

  /**
   * Checks whether the constraint holds.
   * @exception ModelElementException - if the constraint doesn't hold.
   */
  protected void concreteCheck() throws ModelElementException
  {
    Iterator constrElts = getConstrainedElements().iterator();
    while (constrElts.hasNext())
    {
      checkElement((ModelElement) constrElts.next(), this);
    }
  }

  /**
   * Check the constraint for one ModelElement.
   * @param element - the element to check.
   * @param root - the property container root to check against.
   * @exception ModelElementException - if the constraint doesn't hold.
   */
  private void checkElement(ModelElement element, Namespace root) throws ModelElementException 
  {
    if (element instanceof Namespace)
    {
      checkNamespace((Namespace) element, root);
      return;
    }
    else
    {
      if (root.getOwnedElements().size() != 0)
        throw new ModelElementException("Constraint " + getName() + " violated",
          this, Log.ERROR);
    }
  }

  /**
   * Check the constraint for one Namespace.
   * @param namespace - the namespace to check.
   * @param root - the property container root to check against.
   * @exception ModelElementException - if the constraint doesn't hold.
   */
  private void checkNamespace(Namespace namespace, Namespace root) throws ModelElementException 
  {
    Iterator ownedElts = root.getOwnedElements().iterator();
    while (ownedElts.hasNext())
    {
      ModelElement elt = (ModelElement) ownedElts.next();
      if (!namespace.isRegistered(elt.getName()))
        throw new ModelElementException("Constraint " + getName() + " violated",
          this, Log.ERROR);
      if (elt instanceof Namespace)
        checkElement(namespace.getOwnedElement(elt.getName()), (Namespace) elt);
    }
  }
}
