/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: DefaultNamespace.java,v 1.5 2004/04/28 16:30:08 dwagelaa Exp $
 * $Log: DefaultNamespace.java,v $
 * Revision 1.5  2004/04/28 16:30:08  dwagelaa
 * rearranged EPS export under ExportAction
 *
 * Revision 1.4  2004/04/27 16:43:58  dwagelaa
 * + added drag-n-drop
 * + added copy/paste
 * + added general copying operation
 * - removed examples (in separate module now)
 *
 */

package be.ac.vub.cocompose.lang.impl.core;

import java.util.Enumeration;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.Vector;
import java.util.Hashtable;
import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.ModelVisitor;
import be.ac.vub.cocompose.lang.Properties;
import be.ac.vub.cocompose.lang.core.Namespace;
import be.ac.vub.cocompose.lang.core.ModelElement;
import be.ac.vub.cocompose.log.Log;

/**
 * Default container and namespace mechanism for all CoCompose language elements.
 * @version 0.2.9, 27/04/2004
 * @since 0.2.0
 * @author Dennis Wagelaar 
 */
abstract public class DefaultNamespace extends DefaultModelElement implements Namespace
{
  private Vector ownedElements = new Vector();
  private Hashtable ownedNames = new Hashtable();
  private HashSet ownedIds = new HashSet();

  /**
   * Accepts a visitor.
   * @param modelVisitor The visitor to accept.
   */
  public void accept(ModelVisitor modelVisitor) throws ModelElementException 
  {
    modelVisitor.visitNamespace(this);
  }

  /**
   * Adds an element to this namespace.
   * @param modelElement - the element to add.
   * @exception ModelElementException - if the name of the element is already used within this namespace. 
   */
  public void addOwnedElement(ModelElement modelElement) throws ModelElementException
  {
    addOwnedElement(-1, modelElement);
  }

  /**
   * Adds an element to this namespace.
   * @param index - the index at which to add the element.
   * @param modelElement - the element to add.
   * @exception ModelElementException - if the name of the element is already used within this namespace. 
   */
  public void addOwnedElement(int index, ModelElement modelElement) throws ModelElementException
  {
    if (modelElement.getName() == null)
    {
      int serial = ownedElements.size();
      while (isRegistered("ModelElement" + serial))
        serial++;
      modelElement.setName("ModelElement" + serial);
    }
    getOwnedIds().add(getOwnedElementId(modelElement));
    register(modelElement);
    if (index > -1)
      ownedElements.add(index, modelElement);
    else
      ownedElements.add(modelElement);
    modelElement.setNamespace(this);
    fireStructureChange(Properties.ID_OWNED_ELEMENTS, modelElement);
  }

  /**
   * Removes a element from this namespace.
   * @param modelElement - the element to remove. 
   */
  public void removeOwnedElement(ModelElement modelElement)
  {
    ownedElements.remove(modelElement);
    unregister(modelElement.getName());
    getOwnedIds().remove(modelElement.getId());
    fireStructureChange(Properties.ID_OWNED_ELEMENTS, modelElement);
  }

  /**
   * Returns the elements within this namespace.
   * @return a copy of the List of owned elements.
   */
  public List getOwnedElements()
  {
    return (List) ownedElements;
  }

  /**
   * Returns the elements of a given class within this namespace.
   * @param elementClass the element class.
   * @return the List of owned elements.
   */
  public List getOwnedElements(Class elementClass)
  {
    Vector filteredElements = new Vector();
    Enumeration elements = ownedElements.elements();
    while (elements.hasMoreElements())
    {
      Object element = elements.nextElement();
      if (elementClass.isInstance(element))
        filteredElements.add(element);
    }
    return filteredElements;
  }

  /**
   * Returns the elements of a given class within this namespace and nested namespaces.
   * @param elementClass the element class.
   * @return the List of owned elements.
   */
  public List getDeepOwnedElements(Class elementClass)
  {
    Vector filteredElements = new Vector();
    Enumeration elements = ownedElements.elements();
    while (elements.hasMoreElements())
    {
      Object element = elements.nextElement();
      if (elementClass.isInstance(element))
        filteredElements.add(element);
      if (element instanceof Namespace)
        filteredElements.addAll(((Namespace) element).getDeepOwnedElements(elementClass));
    }
    return filteredElements;
  }

  /**
   * Retrieves an owned element by its name.
   * @param name - the name of the element to retrieve.
   * @return the element with the given name or null if it doesn't exist. 
   */
  public ModelElement getOwnedElement(String name)
  {
    ModelElement ownedElement = (ModelElement) ownedNames.get(name);
//    if (ownedElement != null)
//      return ownedElement;
//    int index = name.indexOf('.');
//    if (index != -1)
//    {
//      String topname = name.substring(0, index);
//      ModelElement subElement = (ModelElement) ownedNames.get(topname);
//      if (subElement instanceof Namespace)
//      {
//        String subname = name.substring(index + 1);
//        ownedElement = ((Namespace) subElement).getOwnedElement(subname);
//      }
//    }
    return ownedElement;
  }

  /**
   * Retrieves an owned element in this namespace or a nested namespace.
   * @param copy - the (remote) copy of the element to retrieve.
   * @return the owned element or null if it doesn't exist. 
   */
  public ModelElement getDeepOwnedElement(ModelElement copy)
  {
    Namespace ns = copy.getNamespace();
    if (ns == null)
      return null;
    if (ns.getNamespace() != null)
    {
      ModelElement element = getDeepOwnedElement(ns);
      if (element instanceof Namespace)
        return ((Namespace) element).getOwnedElement(copy.getName());
      else
        return null;
    }
    return getOwnedElement(copy.getName());      
  }

  /**
   * Checks whether a name is already registered within this namespace.
   * @param name - the name to check.
   * @return true if and only if the name is already registered. 
   */
  public boolean isRegistered(String name)
  {
    return ownedNames.containsKey(name);
  }

  /**
   * Registers an element name within this namespace.
   * @param oldName - the name to change.
   * @param newName - the name to change into.
   * @exception ModelElementException - if the new name is already used or if the old name does not exist in this namespace.
   */
  public void registerNameChange(String oldName, String newName) throws ModelElementException
  {
    if (ownedNames.containsKey(newName))
      throw new ModelElementException("Name " + newName +
        " not unique within namespace " + getName(), this,
        Log.ERROR);
    if (!ownedNames.containsKey(oldName))
      throw new ModelElementException("Name " + oldName +
        " does not exist within namespace " + getName(), this,
        Log.ERROR);
    Object element = ownedNames.get(oldName);
    ownedNames.remove(oldName);
    ownedNames.put(newName, element);
  }

  /**
   * Returns a new Id for the given ModelElement.
   * @param modelElement - the element to get the Id for.
   * @return The element Id or null. 
   */
  public String getOwnedElementId(ModelElement modelElement)
  {
    Set ownedIds = getOwnedIds();
    int subid = 0;
    while (ownedIds.contains(Integer.toString(subid)))
    {
      subid++;
    }
    return Integer.toString(subid);
  }

  /**
   * Returns the global ids for the root namespace.
   * @return the global ids for the root namespace.
   */  
  public Set getOwnedIds()
  {
    if (getNamespace() != null)
      return getNamespace().getOwnedIds();
    return ownedIds;
  }

  /**
   * Translates the given ownedElements index to an otherList index.
   * @param index - the index to translate.
   * @param otherList - the other list to which the index should be translated.
   * @return the translated index.
   */
  protected int translateIndex(int index, List otherList)
  {
    return translateIndex(index, ownedElements, otherList);
  }

  /**
   * Translates the given ownedElements index to an otherList index.
   * @param index - the index to translate.
   * @param sourceList - the list to which the index refers.
   * @param otherList - the other list to which the index should be translated.
   * @return the translated index.
   */
  protected int translateIndex(int index, List sourceList, List otherList)
  {
    if (index < 1)
      return index; 
    Object element;
    do
    {
      index--;
      element = sourceList.get(index);
    }
    while (!(otherList.contains(element)) && (index > 0));
    if (index > 0)
    {
      index = otherList.indexOf(element) + 1;
    }
    return index;
  }

  /**
   * Registers an element name within this namespace.
   * @exception ModelElementException - if the name is already used in this namespace.
   * @param element - the element to register.
   */
  private void register(ModelElement element) throws ModelElementException
  {
    if (ownedNames.containsKey(element.getName()))
    {
      throw new ModelElementException("Name " + element.getName() +
        " not unique within namespace " + getName(), this,
        Log.ERROR);
    }
    ownedNames.put(element.getName(), element);
  }

  /**
   * Unregisters a name from this namespace.
   * @param name - the name to unregister. 
   */
  private void unregister(String name)
  {
    ownedNames.remove(name);
  }
}
