/**
 * Title:        CoCompose2
 * Copyright:    Copyright (c) 2003, 2004
 * Author:       Dennis Wagelaar
 * Organisation: Vrije Universiteit Brussel
 * Description:  CoCompose2 Model Driven Design Tool
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 * 
 * $Id: UndeleteElement.java,v 1.3 2004/04/26 17:39:41 dwagelaa Exp $
 * $Log: UndeleteElement.java,v $
 * Revision 1.3  2004/04/26 17:39:41  dwagelaa
 * Changed the defaultConcept property into the defaultElement property, which is a RefinedElement.
 *
 */

package be.ac.vub.cocompose.operations;

import be.ac.vub.cocompose.lang.core.ModelElement;
import be.ac.vub.cocompose.lang.core.RefinedElement;
import be.ac.vub.cocompose.lang.DefaultSuperModelVisitor;
import be.ac.vub.cocompose.lang.ModelElementException;
import be.ac.vub.cocompose.lang.core.Relationship;

/**
 * Undo for a DeleteElement operation.
 * @author Dennis Wagelaar 
 * @version 0.2.9, 26/04/2004
 * @since 0.2.0
 * @see DeleteElement
 */
public class UndeleteElement extends DefaultSuperModelVisitor 
{
  private ModelElement deletedElement = null;
  private int elementIndex = -1;
  private int sourceIndex = -1;
  private int targetIndex = -1;

  /**
   * Gets the deleted element to undelete.
   * @return the deleted element.
   */
  public ModelElement getDeletedElement()
  {
    return deletedElement;
  }

  /**
   * Sets the deleted element to undelete.
   * @param deletedElement - the deleted element.
   */
  public void setDeletedElement(ModelElement deletedElement)
  {
    this.deletedElement = deletedElement;
  }

  /**
   * Visits a ModelElement.
   * @param modelElement - the element to visit.
   * @exception <{be.ac.vub.cocompose.lang.ModelElementException}>
   */
  public void visitModelElement(ModelElement modelElement) throws ModelElementException
  {
    super.visitModelElement(modelElement);
    if (modelElement.getNamespace() != null)
      modelElement.getNamespace().addOwnedElement(getElementIndex(), modelElement);
  }

  /**
   * Visits a Relationship.
   * @param relationship - the element to visit.
   * @exception <{ModelElementException}>
   */
  public void visitRelationship(Relationship relationship) throws ModelElementException
  {
    super.visitRelationship(relationship);
    relationship.getSourceParticipant().addRelationship(getSourceIndex(), relationship);
    relationship.getTargetParticipant().addRelationship(getTargetIndex(), relationship);
  }

  /**
   * Visits a RefinedElement.
   * @param refinedElement - the element to visit.
   * @exception ModelElementException
   */
  public void visitRefinedElement(RefinedElement refinedElement) throws ModelElementException
  {
    super.visitRefinedElement(refinedElement);
    if (refinedElement.getDefaultElementOf() != null)
    {
      refinedElement.getDefaultElementOf().setDefaultElement(refinedElement);
    }
  }

  /**
   * Gets the element participant index at which a relationship was deleted.
   * @return the index.
   */
  public int getElementIndex()
  {
    return elementIndex;
  }

  /**
   * Sets the element participant index at which a relationship was deleted.
   * @param elementIndex - the index.
   */
  public void setElementIndex(int elementIndex)
  {
    this.elementIndex = elementIndex;
  }

  /**
   * Gets the source participant index at which a relationship was deleted.
   * @return the index.
   */
  public int getSourceIndex()
  {
    return sourceIndex;
  }

  /**
   * Sets the source participant index at which a relationship was deleted.
   * @param sourceIndex - the index.
   */
  public void setSourceIndex(int sourceIndex)
  {
    this.sourceIndex = sourceIndex;
  }

  /**
   * Gets the target participant index at which a relationship was deleted.
   * @return the index.
   */
  public int getTargetIndex()
  {
    return targetIndex;
  }

  /**
   * Sets the target participant index at which a relationship was deleted.
   * @param targetIndex - the index.
   */
  public void setTargetIndex(int targetIndex)
  {
    this.targetIndex = targetIndex;
  }
}
